#ifndef PORTS_SYS_CLOCKTIMER_H
#define PORTS_SYS_CLOCKTIMER_H

#include <stdlib.h>
#include <time.h>
#include <sys/time.h>

typedef struct timeval PORTS_Clock0_t;

struct PORTS_T0_t {
  int count;
  PORTS_Clock0_t sum;
  PORTS_Clock0_t clk;
};

typedef struct PORTS_T0_t *PORTS_Timer0_t;

#define PORTS_Num_ClockTimer() 1

#define PORTS_ClockTimerTerm() PORTS_CLOCKTIMER_SUCCESS

#if defined(__cplusplus)

static inline void PORTS_TimerStart0(PORTS_Timer0_t t) {
  gettimeofday(&((t)->clk),0);
}

static inline void PORTS_TimerStop0(PORTS_Timer0_t t) {
  struct timeval x;

  gettimeofday(&x, 0);
  t->count++;
  t->sum.tv_sec += x.tv_sec - t->clk.tv_sec;
  t->sum.tv_usec += x.tv_usec - t->clk.tv_usec;
}

static inline void PORTS_Clock0(PORTS_Clock0_t& c) {
  gettimeofday(&c, 0);
}

static inline double PORTS_ClockToSeconds0 (PORTS_Clock0_t c) {
  return (c.tv_sec + c.tv_usec * 1.0e-6);
}

#else

#define PORTS_TimerStart0(t) gettimeofday(&((t)->clk),0)

#define PORTS_TimerStop0(t) \
  do { \
    struct timeval x; \
    gettimeofday(&x,0); \
    (t)->count++; \
    (t)->sum.tv_sec += x.tv_sec - (t)->clk.tv_sec; \
    (t)->sum.tv_usec += x.tv_usec - (t)->clk.tv_usec; \
  } while (0)

#define PORTS_Clock0(c) gettimeofday(&c,0)

#define PORTS_ClockToSeconds0(c) (c.tv_sec + c.tv_usec * 1.0e-6)

#endif

#define PORTS_TimerInit0(t) PORTS_TimerInitInternal0(&t)

#ifdef __cplusplus
extern "C" {
#endif

extern int PORTS_ClockTimerInit(int *argc, char **argv);

extern int PORTS_TimerInitInternal0(PORTS_Timer0_t *t);

extern int PORTS_TimerDestroy0(PORTS_Timer0_t t);

extern int PORTS_TimerReset0(PORTS_Timer0_t t);

extern double PORTS_TimerElapsed0(PORTS_Timer0_t t, int *c);

#ifdef __cplusplus
}
#endif
#endif
